require 'csv'
require 'geocoder'

require 'columbus3/v900track/v900waypoint'

class V900Track
  attr_reader :filename

  def initialize hash 
    @filename = hash[:filename]
    if hash[:empty]
      @data = []
    else
      read filename
    end
  end

  def read filename
    @filename = filename
    @data = CSV::read(filename, headers: true).each.map { |item| V900Waypoint.new(item) }
  end

  def save
    File.open(@filename, 'w') {|f| f.write(to_s) }
  end

  def add v900_waypoint
    @data << v900_waypoint
  end

  def size
    @data.size
  end

  # an alias for size
  def points
    @data.size
  end

  def range
    (0..self.size - 1)
  end

  # to enumerate (till I can solve the each issue)
  def to_a
    @data
  end

  # http://stackoverflow.com/questions/19226821/iterator-array-each-as-return-value-of-method-in-ruby
  def each(&block)
    @data.each(&block)
  end
 
  def get i
    self[i]
  end

  def [](i)
    @data[i]
  end

  def first
    @data[0]
  end

  def last
    @data[size - 1]
  end

  # statistics

  def start_date
    first.time
  end

  def end_date
    last.time
  end

  def duration
    last.time - first.time
  end

  def min_speed
    get :<, :speed
  end

  def max_speed
    get :>, :speed
  end

  def min_height
    get :<, :height
  end

  def max_height
    get :>, :height
  end

  def min_speed_idx
    get_idx :<, :speed
  end

  def max_speed_idx
    get_idx :>, :speed
  end

  def min_height_idx
    get_idx :<, :height
  end

  def max_height_idx
    get_idx :>, :height
  end

  def start_location
    Geocoder.address(first.lat_lon)
  end

  def end_location
    Geocoder.address(last.lat_lon)
  end

  def bearing
    Geocoder::Calculations.compass_point(Geocoder::Calculations.bearing_between(first.lat_lon, last.lat_lon))
  end

  def distance_aerial
    Geocoder::Calculations.distance_between(first.lat_lon, last.lat_lon, :units => :km)
  end

  def distance
    distance = 0
    range.each do |i|
      distance += Geocoder::Calculations.distance_between(self[i].lat_lon, self[i-1].lat_lon, :units => :km)
    end
    distance
  end

  # return a hash with all the metadata
  # ... it could be a nice metamethod, creating a key for each public methods
  def metadata
    metadata = Hash.new

    [:start_date, :end_date, :duration, :size, :start_location, :end_location, :min_speed, :max_speed, :min_height, :max_height, :bearing, :distance, :distance_aerial].map { |x| metadata[x] = self.send(x) }

    metadata
  end

  # a slightly more robust implementation calls data["INDEX"] in the same order
  # in which the header is printed
  def to_s
    string = "INDEX,TAG,DATE,TIME,LATITUDE N/S,LONGITUDE E/W,HEIGHT,SPEED,HEADING,VOX\n"
    @data.each do |data|
      string << data.to_s + "\n"
    end
    string
  end

  private

  def get comparison, method
    current = self[0].send(method) # initialize max or min to the first value in the file
    range.each do |i|
      current = self[i].send(method) if self[i].send(method).send(comparison, current)
    end
    current
  end

  def get_idx comparison, method
    current = self[0].send(method) # initialize max or min to the first value in the file
    idx = 0
    range.each do |i|
      if self[i].send(method).send(comparison, current) then
        current = self[i].send(method)
        idx = i
      end
    end
    idx
  end

end

